%% --------------------------------- Image Aesthetic Quality --------------------------------- %%
%% ImageAestheticQuality : Estimate the  aesthetic features for photo aesthetics assessment    %% 
%% input variables                                                                             %%
%% imagepath : the path of the photos                                                          %%
%% label     : the label of feature vectors (1 or -1), in order to be compatible with LIBSVM   %%
%% example of run -> ImageAestheticQuality( 'images\', 1) 								       %%		
%% Citation  : "E. Mavridaki, V. Mezaris, "A comprehensive aesthetic quality assessment method %%
%% for natural image using basic rules of photography, Proc. IEEE International Conference on %%
%% Image Processing (ICIP 2015), Quebec, Canada, September 2015."                              %%                 
%% ------------------------------------------------------------------------------------------- %%

function ImageAestheticQuality( imagepath, label)
% start up timer
tic; 

%% Create .txt files where the corresponding easthetic features will be stored
fidSimple = fopen('Results\Simplicity.txt','w');                  % Simplicity
fidColor = fopen('Results\Colorfulness.txt','w');                 % Colorfulness
fidBlur = fopen('Results\Sharpness.txt','w');                     % Sharpness
fidPattern = fopen('Results\Pattern.txt','w');                    % Pattern
fidCenter = fopen('Results\Center.txt','w');                      % Composition (Center)
fidRuleOfThirds = fopen('Results\RuleOfThirds.txt','w');          % Composition (Rule of thirds)
fidLandscape = fopen('Results\Landscape.txt','w');                % Composition (Landscape)
% fidNames = fopen('Names.txt','w');                              % comment if parallel processing runs
%--------------------------------------------------

%% Initialization
% open matlabpool for parallel processing
matlabpool;                                                                        % comment if you don't want parallel processing

% Add and setup vlfeat
addpath('vlfeat-0.9.18\','vlfeat-0.9.18\toolbox\');
vl_setup;
warning off all;

% Go to the composition path to set up paths and initial parameters
CompositionPath = [pwd '\Composition\'];
cd(CompositionPath)
startup
% Return to the main directory
cd('..')

% Read the folder where the images are stored
% d = dir(imagepath);                  % read and list all files from input path  % uncomment : if you have multiple image formats (.png, .jpg, etc)
% d(1:2) = [];                         % remove . and .. from list                % uncomment : if you have multiple image formats (.png, .jpg, etc)
d = dir(fullfile(imagepath, '*.jpg')); % read only .jpg file image format         % comment : if you don't have multiple image formats (.png, .jpg, etc)
numOfImages = size(d,1);               % store the number of images
%--------------------------------------------------

%% Read all images from the corresponding repository
parfor i = 1 : numOfImages
    
    % Read the image
    rawName = d(i).name;
    
    % Check if the input file is an image or a thumb file
    %     [~, ~, extension] = fileparts(rawName);                                 % uncomment : if you have multiple image formats (.png, .jpg, etc)
    %     if ~(extension=='.db')                                                  % uncomment :if you have multiple image formats (.png, .jpg, etc)
    
    image = imread([imagepath rawName]);
    %     fprintf(fidNames,'%s \n', rawName);                                     % comment : if parallel processing runs
    
    %  Convert a grayscale image to RGB colorspace
    if size(image,3) == 1
        image = gray2rgb(image);
    end
    
    % Remove image Frame / Border to improve image segmentation
    cur_image(i) = {removeFrame(image)}; %#ok<PFOUS>
    
    % Convert a color image ( RGB) to grayscale
    GRAYimage{i} = cur_image{i};
    if(size(image,3)~=1)
        GRAYimage{i} = rgb2gray(cur_image{i});
    end
    
    % Resize input image into the range of 640 * 480 px or 480 * 640 px
    resizedImage{i} = resizeImage(cur_image{i});
    
    % Convert a color image to HSV and Lab colorspace
    HSVresizedImage{i} = rgb2hsv(resizedImage{i});
    LabresizedImage{i} = rgb2lab(resizedImage{i});
    
end
clear cur_image;
%--------------------------------------------------

%% Photo Aesthetic Assessment
for i = 1 : numOfImages
    %%% Simplicity %%%
    % Read the Simplicity function from the corresponding path
    SimplicityPath = [pwd '\Simplicity\'];
    cd(SimplicityPath)
    display('Simplicity Measure ...');
    SimplicityMeasure(resizedImage{i}, HSVresizedImage{i}, LabresizedImage{i}, fidSimple, label);
    % Return to the main directory
    cd('..')
    %--------------------------------------------------
    
    %%% Colorfulness %%%
    % Read the Colorfulness function from the corresponding path
    ColorfulnessPath = [pwd '\Colorfulness\'];
    cd(ColorfulnessPath)
    display('Colorfulness Measure ...');
    ColorfulnessMeasure(resizedImage{i}, HSVresizedImage{i}, LabresizedImage{i}, fidColor, label);
    % Return to the main directory
    cd('..')
    %--------------------------------------------------
    
    %%% Sharpness %%%
    % Read the Sharpness function from the corresponding path
    SharpnessPath = [pwd '\Sharpness\'];
    cd(SharpnessPath)
    display('Sharpness Measure ...');
    SharpnessMeasure(GRAYimage{i}, fidBlur, label);
    % Return to the main directory
    cd('..')
    %--------------------------------------------------
    
    %%% Pattern %%%
    % Read the Pattern function from the corresponding path
    PatternPath = [pwd '\Pattern\'];
    cd(PatternPath)
    display('Pattern Measure ...');
    PatternMeasure(GRAYimage{i}, fidPattern, label);
    % Return to the main directory
    cd('..')
    %--------------------------------------------------
    
    %%% Composition %%%
    % Read the Composition function from the corresponding path
    CompositionPath = [pwd '\Composition\'];
    cd(CompositionPath)
    display('Composition Measures ...');
    CompositionMeasure(resizedImage{i}, HSVresizedImage{i}, fidCenter, fidRuleOfThirds, fidLandscape);
    % Return to the main directory
    cd('..')
    %--------------------------------------------------
end

%% Close .txt files
fclose(fidSimple);
fclose(fidColor);
fclose(fidBlur);
fclose(fidPattern);
fclose(fidCenter);
fclose(fidRuleOfThirds);
fclose(fidLandscape);
% fclose(fidNames);                  % comment : if you use parallel processing
%--------------------------------------------------

%% Create feature files in form compatible with svm
fileCenter = 'Results\Center.txt';
fileRuleOfThirds = 'Results\RuleOfThirds.txt';
fileLandscape = 'Results\Landscape.txt';
CreateSVMfiles(fileCenter, fileCenter, label);
CreateSVMfiles(fileRuleOfThirds, fileRuleOfThirds, label);
CreateSVMfiles(fileLandscape, fileLandscape, label);
ConcatenateVectors( 'Results\Composition.txt', label , 0)
%--------------------------------------------------

% Close matlabpool
matlabpool close;                   % comment : if you don't want parallel processing
toc
end

